@@ -1,7 +1,7 @@
 /*
 OpenIO SDS meta2v2
 Copyright (C) 2014 Worldine, original work as part of Redcurrant
-Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage
+Copyright (C) 2015-2016 OpenIO, as part of OpenIO Software Defined Storage
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU Affero General Public License as
@@ -28,9 +28,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <events/oio_events_queue.h>
 
-#include <meta0v2/meta0_remote.h>
-#include <meta0v2/meta0_utils.h>
-
 #include <meta2v2/generic.h>
 #include <meta2v2/autogen.h>
 #include <meta2v2/meta2v2_remote.h>
@@ -61,31 +58,28 @@ enum m2v2_open_type_e
 #define M2V2_OPEN_STATUS    0xF00
 };
 
-struct m2_prepare_data {
+struct m2_prepare_data
+{
 	gint64 max_versions;
 	gint64 quota;
 	gint64 size;
 	gchar storage_policy[LIMIT_LENGTH_STGPOLICY];
 };
 
-static void
-_append_url (GString *gs, struct oio_url_s *url)
+static void _meta2_backend_force_prepare_data(struct meta2_backend_s *m2b,
+		const gchar *key, struct sqlx_sqlite3_s *sq3);
+
+static enum m2v2_open_type_e
+_mode_masterslave(guint32 flags)
 {
-	void _append (const char *n, const char *v) {
-		if (v)
-			oio_str_gstring_append_json_pair(gs, n, v);
-		else
-			g_string_append_printf(gs, "\"%s\":null", n);
-	}
-	_append ("ns", oio_url_get(url, OIOURL_NS));
-	g_string_append_c (gs, ',');
-	_append ("account", oio_url_get(url, OIOURL_ACCOUNT));
-	g_string_append_c (gs, ',');
-	_append ("user", oio_url_get(url, OIOURL_USER));
-	g_string_append_c (gs, ',');
-	_append ("type", oio_url_get(url, OIOURL_TYPE));
-	g_string_append_c (gs, ',');
-	_append ("id", oio_url_get(url, OIOURL_HEXID));
+	return (flags & M2V2_FLAG_MASTER)
+		? M2V2_OPEN_MASTERONLY : M2V2_OPEN_MASTERSLAVE;
+}
+
+static enum m2v2_open_type_e
+_mode_readonly(guint32 flags)
+{
+	return M2V2_OPEN_ENABLED|M2V2_OPEN_FROZEN|_mode_masterslave(flags);
 }
 
 static gint64
@@ -184,11 +178,11 @@ meta2_backend_get_local_addr(struct meta2_backend_s *m2)
 GError *
 meta2_backend_init(struct meta2_backend_s **result,
 		struct sqlx_repository_s *repo, const gchar *ns,
-		struct grid_lbpool_s *glp, struct hc_resolver_s *resolver)
+		struct oio_lb_s *lb, struct hc_resolver_s *resolver)
 {
 	EXTRA_ASSERT(result != NULL);
 	EXTRA_ASSERT(repo != NULL);
-	EXTRA_ASSERT(glp != NULL);
+	EXTRA_ASSERT(lb != NULL);
 	EXTRA_ASSERT(resolver != NULL);
 
 	if (!*ns || strlen(ns) >= LIMIT_LENGTH_NSNAME)
@@ -198,7 +192,7 @@ meta2_backend_init(struct meta2_backend_s **result,
 	g_strlcpy(m2->ns_name, ns, sizeof(m2->ns_name));
 	m2->type = NAME_SRVTYPE_META2;
 	m2->repo = repo;
-	m2->lb = glp;
+	m2->lb = lb;
 	m2->policies = service_update_policies_create();
 	g_mutex_init(&m2->nsinfo_lock);
 	m2->flag_precheck_on_generate = TRUE;
@@ -273,36 +267,6 @@ meta2_backend_get_nsinfo (struct meta2_backend_s *m2)
 	return out;
 }
 
-GError*
-meta2_backend_poll_service(struct meta2_backend_s *m2,
-		const gchar *type, struct service_info_s **si)
-{
-	struct grid_lb_iterator_s *iter;
-
-	EXTRA_ASSERT(m2 != NULL);
-	EXTRA_ASSERT(type != NULL);
-	EXTRA_ASSERT(si != NULL);
-
-	if (!(iter = grid_lbpool_get_iterator(m2->lb, type)))
-		return NEWERROR(CODE_SRVTYPE_NOTMANAGED, "no such service");
-
-	struct lb_next_opt_ext_s opt_ext;
-	memset(&opt_ext, 0, sizeof(opt_ext));
-	opt_ext.req.distance = 0;
-	opt_ext.req.max = 1;
-	opt_ext.req.duplicates = TRUE;
-	opt_ext.req.stgclass = NULL;
-	opt_ext.req.strict_stgclass = TRUE;
-
-	struct service_info_s **siv = NULL;
-	if (!grid_lb_iterator_next_set2(iter, &siv, &opt_ext))
-		return NEWERROR(CODE_SRVTYPE_NOTMANAGED, "no service available");
-
-	*si = service_info_dup(siv[0]);
-	service_info_cleanv(siv, FALSE);
-	return NULL;
-}
-
 gboolean
 meta2_backend_initiated(struct meta2_backend_s *m2)
 {
@@ -340,6 +304,12 @@ m2_to_sqlx(enum m2v2_open_type_e t)
 	return result;
 }
 
+static gboolean
+_is_container_initiated(struct sqlx_sqlite3_s *sq3)
+{
+	return sqlx_admin_has(sq3, META2_INIT_FLAG);
+}
+
 static void
 m2b_close(struct sqlx_sqlite3_s *sq3)
 {
@@ -391,7 +361,7 @@ m2b_open(struct meta2_backend_s *m2, struct oio_url_s *url,
 
 	sq3->no_peers = how & (M2V2_OPEN_LOCAL|M2V2_OPEN_NOREFCHECK);
 
-	// XXX If the container is being deleted, this is sad ...
+	// If the container is being deleted, this is sad ...
 	// This MIGHT happen if a cache is present (and this is the
 	// common case for m2v2), because the deletion will happen
 	// when the base exit the cache.
@@ -399,9 +369,21 @@ m2b_open(struct meta2_backend_s *m2, struct oio_url_s *url,
 	// is closed with an instruction to exit the cache immediately.
 	// TODO FIXME this is maybe a good place for an assert().
 	if (sq3->deleted) {
-		err = NEWERROR(CODE_CONTAINER_FROZEN, "destruction pending");
 		m2b_close(sq3);
-		return err;
+		return NEWERROR(CODE_CONTAINER_FROZEN, "destruction pending");
+	}
+
+	/* M2V2_OPEN_AUTOCREATE is only used when creating the container. If not
+	 * specified, we expect the container to also be "softly" initiated.
+	 * Non-LOCAL accesses are made by the functional requests, and by no
+	 * other, excepted by the DESTROY request. */
+	if (!(how & M2V2_OPEN_AUTOCREATE) &&
+			M2V2_OPEN_LOCAL == (how & M2V2_OPEN_REPLIMODE)) {
+		if (!_is_container_initiated(sq3)) {
+			m2b_close(sq3);
+			return NEWERROR(CODE_CONTAINER_NOTFOUND,
+					"container created but not initiated");
+		}
 	}
 
 	// Complete URL with full VNS and container name
@@ -473,7 +455,10 @@ meta2_backend_has_container(struct meta2_backend_s *m2,
 	struct sqlx_sqlite3_s *sq3 = NULL;
 	err = m2b_open(m2, url, M2V2_OPEN_LOCAL, &sq3);
 	if (NULL == err) {
-		if (!sqlx_admin_has(sq3, META2_INIT_FLAG))
+		/* The base is used in LOCAL mode, and with that option the INIT
+		 * flag is not checked. As an exception, we want the check to be
+		 * performed here. */
+		if (!_is_container_initiated(sq3))
 			err = NEWERROR(CODE_CONTAINER_NOTFOUND,
 					"Container created but not initiated");
 		m2b_close(sq3);
@@ -482,7 +467,7 @@ meta2_backend_has_container(struct meta2_backend_s *m2,
 }
 
 static GError *
-_isempty (struct sqlx_sqlite3_s *sq3)
+_check_if_container_empty (struct sqlx_sqlite3_s *sq3)
 {
 	GError *err = NULL;
 	sqlite3_stmt *stmt = NULL;
@@ -518,9 +503,9 @@ meta2_backend_container_isempty (struct meta2_backend_s *m2,
 	GRID_DEBUG("ISEMPTY(%s)", oio_url_get(url, OIOURL_WHOLE));
 
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	GError *err = m2b_open(m2, url, M2V2_OPEN_MASTERSLAVE, &sq3);
-	if (NULL == err) {
-		err = _isempty (sq3);
+	GError *err = m2b_open(m2, url, _mode_masterslave(0), &sq3);
+	if (!err) {
+		err = _check_if_container_empty (sq3);
 		m2b_close(sq3);
 	}
 	return err;
@@ -537,8 +522,7 @@ meta2_backend_get_max_versions(struct meta2_backend_s *m2b,
 	EXTRA_ASSERT(url != NULL);
 	EXTRA_ASSERT(result != NULL);
 
-	err = m2b_open(m2b, url, M2V2_OPEN_MASTERSLAVE
-			|M2V2_OPEN_ENABLED|M2V2_OPEN_FROZEN, &sq3);
+	err = m2b_open(m2b, url, _mode_readonly(0), &sq3);
 	if (!err) {
 		*result = _maxvers(sq3, m2b);
 		m2b_close(sq3);
@@ -548,7 +532,7 @@ meta2_backend_get_max_versions(struct meta2_backend_s *m2b,
 }
 
 static GError *
-_create_container_init_phase(struct sqlx_sqlite3_s *sq3,
+_init_container(struct sqlx_sqlite3_s *sq3,
 		struct oio_url_s *url, struct m2v2_create_params_s *params)
 {
 	GError *err = NULL;
@@ -565,6 +549,8 @@ _create_container_init_phase(struct sqlx_sqlite3_s *sq3,
 	}
 	if (!err) {
 		m2db_set_ctime (sq3, oio_ext_real_time());
+		m2db_set_size(sq3, 0);
+		m2db_set_obj_count(sq3, 0);
 		sqlx_admin_init_i64(sq3, META2_INIT_FLAG, 1);
 	}
 	if (!err && params->properties) {
@@ -600,6 +586,8 @@ meta2_backend_create_container(struct meta2_backend_s *m2,
 		open_mode = M2V2_OPEN_LOCAL|M2V2_OPEN_NOREFCHECK;
 	else
 		open_mode = M2V2_OPEN_MASTERONLY;
+
+	/* The AUTOCREATE flag disables the INIT check */
 	open_mode |= M2V2_OPEN_AUTOCREATE;
 
 	err = m2b_open(m2, url, open_mode, &sq3);
@@ -607,24 +595,25 @@ meta2_backend_create_container(struct meta2_backend_s *m2,
 		if (sqlx_admin_has(sq3, META2_INIT_FLAG))
 			err = NEWERROR(CODE_CONTAINER_EXISTS, "Container already initiated");
 		else {
-			err = _create_container_init_phase(sq3, url, params);
+			err = _init_container(sq3, url, params);
 			if (err) {
 				m2b_destroy(sq3);
 				return err;
 			}
+
+			/* Fire an event to notify the world this container exists */
 			const enum election_status_e s = sq3->election;
 			if (!params->local && m2->notifier && (!s || s == ELECTION_LEADER)) {
-				GString *gs = g_string_new ("{");
-				g_string_append (gs, "\"event\":\"" META2_EVENTS_PREFIX
-						".container.new\"");
-				g_string_append_printf (gs, ",\"when\":%"G_GINT64_FORMAT,
-						oio_ext_real_time());
-				g_string_append (gs, ",\"data\":{");
-				g_string_append (gs, "\"url\":{");
-				_append_url (gs, url);
-				g_string_append (gs, "}}}");
+				GString *gs = oio_event__create (META2_EVENTS_PREFIX".container.new", url);
+				g_string_append (gs, ",\"data\":null}");
 				oio_events_queue__send (m2->notifier, g_string_free (gs, FALSE));
 			}
+
+			/* Reload any cache maybe already associated with the container.
+			 * It happens that the cache sometimes exists because created during a
+			 * M2_PREP that occured after a GETVERS (the meta1 is filled) but before
+			 * the CREATE. */
+			meta2_backend_change_callback(sq3, m2);
 		}
 		m2b_close(sq3);
 	}
@@ -646,7 +635,7 @@ meta2_backend_destroy_container(struct meta2_backend_s *m2,
 		EXTRA_ASSERT(sq3 != NULL);
 
 		if (flush && !force)
-			err = _isempty (sq3);
+			err = _check_if_container_empty (sq3);
 
 		if (!err && flush) {
 			err = m2db_flush_container(sq3->db);
@@ -663,15 +652,8 @@ meta2_backend_destroy_container(struct meta2_backend_s *m2,
 		if (!err) {
 			GString *gs = NULL;
 			if (event && m2->notifier) {
-				gs = g_string_new ("{");
-				g_string_append (gs, "\"event\":\"" META2_EVENTS_PREFIX
-						".container.deleted\"");
-				g_string_append_printf (gs, ",\"when\":%"G_GINT64_FORMAT,
-						oio_ext_real_time());
-				g_string_append (gs, ",\"data\":{");
-				g_string_append (gs, "\"url\":{");
-				_append_url (gs, url);
-				g_string_append (gs, "}}}");
+				gs = oio_event__create (META2_EVENTS_PREFIX ".container.deleted", url);
+				g_string_append (gs, ",\"data\":null}");
 			}
 			m2b_destroy(sq3);
 			if (gs) {
@@ -743,8 +725,7 @@ meta2_backend_list_aliases(struct meta2_backend_s *m2b, struct oio_url_s *url,
 	EXTRA_ASSERT(url != NULL);
 	EXTRA_ASSERT(lp != NULL);
 
-	err = m2b_open(m2b, url, M2V2_OPEN_MASTERSLAVE
-			|M2V2_OPEN_ENABLED|M2V2_OPEN_FROZEN, &sq3);
+	err = m2b_open(m2b, url, _mode_readonly(0), &sq3);
 	if (!err) {
 		err = m2db_list_aliases(sq3, lp, headers, cb, u0);
 		if (!err && out_properties)
@@ -766,8 +747,7 @@ meta2_backend_get_alias(struct meta2_backend_s *m2b,
 	EXTRA_ASSERT(m2b != NULL);
 	EXTRA_ASSERT(url != NULL);
 
-	err = m2b_open(m2b, url, M2V2_OPEN_MASTERSLAVE
-			|M2V2_OPEN_ENABLED|M2V2_OPEN_FROZEN, &sq3);
+	err = m2b_open(m2b, url, _mode_readonly(flags), &sq3);
 	if (!err) {
 		err = m2db_get_alias(sq3, url, flags, cb, u0);
 		m2b_close(sq3);
@@ -776,6 +756,8 @@ meta2_backend_get_alias(struct meta2_backend_s *m2b,
 	return err;
 }
 
+/* TODO(jfs): merge the container's state with the m2_prepare_data and keep
+ *            it cached? */
 static gchar *
 _container_state (struct sqlx_sqlite3_s *sq3)
 {
@@ -791,32 +773,32 @@ _container_state (struct sqlx_sqlite3_s *sq3)
 		sep (gs);
 		oio_str_gstring_append_json_pair(gs, k, v);
 	}
-	void append (GString *gs, const char *k, gchar *v) {
-		append_const (gs, k, v);
-		g_free0 (v);
-	}
 
-	GString *gs = g_string_new("{");
-	append_const (gs, "event", META2_EVENTS_PREFIX ".container.state");
-	append_int64 (gs, "when", oio_ext_real_time());
-	g_string_append (gs, ",\"url\":{");
-	append (gs, "ns", sqlx_admin_get_str(sq3, SQLX_ADMIN_NAMESPACE));
-	append (gs, "account", sqlx_admin_get_str(sq3, SQLX_ADMIN_ACCOUNT));
-	append (gs, "user", sqlx_admin_get_str(sq3, SQLX_ADMIN_USERNAME));
-	append_const (gs, "type", sq3->name.type);
-	g_string_append (gs, "}, \"data\":{");
+	struct oio_url_s *u = sqlx_admin_get_url (sq3);
 
+	/* Patch the SUBTYPE: the URL must represent an object as managed by meta2
+	 * services (and also by end-user APIs). Unlike for sqlx services, the user
+	 * doesn't know anything about the meta2, and never mentions it in URL.
+	 * This is why we strip this from the service type. */
+	if (!strcmp(sq3->name.type, NAME_SRVTYPE_META2))
+		oio_url_set (u, OIOURL_TYPE, "");
+	else if (g_str_has_prefix (sq3->name.type, NAME_SRVTYPE_META2 "."))
+		oio_url_set (u, OIOURL_TYPE, sq3->name.type + sizeof(NAME_SRVTYPE_META2));
+
+	GString *gs = oio_event__create (META2_EVENTS_PREFIX ".container.state", u);
+	g_string_append (gs, ",\"data\":{");
 	append_const (gs, "policy", sqlx_admin_get_str(sq3, M2V2_ADMIN_STORAGE_POLICY));
 	append_int64 (gs, "ctime", m2db_get_ctime(sq3));
 	append_int64 (gs, "bytes-count", m2db_get_size(sq3));
-	append_int64 (gs, "object-count", 0);
+	append_int64 (gs, "object-count", m2db_get_obj_count(sq3));
 	g_string_append (gs, "}}");
 
+	oio_url_clean (u);
 	return g_string_free(gs, FALSE);
 }
 
 static void
-meta2_backend_add_modified_container(struct meta2_backend_s *m2b,
+m2b_add_modified_container(struct meta2_backend_s *m2b,
 		struct sqlx_sqlite3_s *sq3)
 {
 	EXTRA_ASSERT(m2b != NULL);
@@ -824,22 +806,50 @@ meta2_backend_add_modified_container(struct meta2_backend_s *m2b,
 		oio_events_queue__send_overwritable(m2b->notifier,
 				sqlx_admin_get_str(sq3, SQLX_ADMIN_BASENAME),
 				_container_state (sq3));
+
+	gboolean has_peers = FALSE;
+	GError *err = election_has_peers(sq3->manager,
+			sqlx_name_mutable_to_const(&(sq3->name)), FALSE, &has_peers);
+	if (!err && !has_peers) {
+		meta2_backend_change_callback(sq3, m2b);
+	}
+	g_clear_error(&err);
+}
+
+/* TODO(jfs): maybe is there a way to keep this in a cache */
+GError *
+meta2_backend_notify_container_state(struct meta2_backend_s *m2b,
+		struct oio_url_s *url)
+{
+	GError *err = NULL;
+	struct sqlx_sqlite3_s *sq3 = NULL;
+	if (!(err = m2b_open(m2b, url, M2V2_OPEN_MASTERONLY, &sq3))) {
+		m2b_add_modified_container(m2b, sq3);
+		m2b_close(sq3);
+	}
+	return err;
 }
 
 GError*
 meta2_backend_refresh_container_size(struct meta2_backend_s *m2b,
 		struct oio_url_s *url, gboolean recompute)
 {
-    GError *err = NULL;
-    struct sqlx_sqlite3_s *sq3 = NULL;
+	GError *err = NULL;
+	struct sqlx_sqlite3_s *sq3 = NULL;
 
 	EXTRA_ASSERT(m2b != NULL);
 	EXTRA_ASSERT(url != NULL);
 
 	if (!(err = m2b_open(m2b, url, M2V2_OPEN_MASTERONLY, &sq3))) {
-		if (recompute)
-			m2db_set_size(sq3, m2db_get_container_size(sq3->db, FALSE));
-		meta2_backend_add_modified_container(m2b, sq3);
+		if (recompute) {
+			guint64 size = 0u;
+			gint64 count = 0;
+			m2db_get_container_size_and_obj_count(sq3->db, FALSE,
+					&size, &count);
+			m2db_set_size(sq3, size);
+			m2db_set_obj_count(sq3, count);
+		}
+		m2b_add_modified_container(m2b, sq3);
 		m2b_close(sq3);
 	}
 
@@ -852,7 +862,6 @@ meta2_backend_delete_alias(struct meta2_backend_s *m2b,
 {
 	GError *err = NULL;
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	gint64 max_versions;
 
 	EXTRA_ASSERT(m2b != NULL);
 	EXTRA_ASSERT(url != NULL);
@@ -860,7 +869,7 @@ meta2_backend_delete_alias(struct meta2_backend_s *m2b,
 	err = m2b_open(m2b, url, M2V2_OPEN_MASTERONLY|M2V2_OPEN_ENABLED, &sq3);
 	if (!err) {
 		struct sqlx_repctx_s *repctx = NULL;
-		max_versions = _maxvers(sq3, m2b);
+		const gint64 max_versions = _maxvers(sq3, m2b);
 		if (!(err = _transaction_begin(sq3, url, &repctx))) {
 			if (!(err = m2db_delete_alias(sq3, max_versions, url, cb, u0))) {
 				m2db_increment_version(sq3);
@@ -868,7 +877,7 @@ meta2_backend_delete_alias(struct meta2_backend_s *m2b,
 			err = sqlx_transaction_end(repctx, err);
 		}
 		if (!err)
-			meta2_backend_add_modified_container(m2b, sq3);
+			m2b_add_modified_container(m2b, sq3);
 		m2b_close(sq3);
 	}
 
@@ -895,20 +904,16 @@ meta2_backend_put_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 		memset(&args, 0, sizeof(args));
 		args.sq3 = sq3;
 		args.url = url;
-		args.max_versions = _maxvers(sq3, m2b);
-		args.nsinfo = meta2_backend_get_nsinfo(m2b);
-		args.lbpool = m2b->lb;
+		args.ns_max_versions = m2b_max_versions(m2b);
 
 		if (!(err = _transaction_begin(sq3, url, &repctx))) {
 			if (!(err = m2db_put_alias(&args, in, out_deleted, out_added)))
 				m2db_increment_version(sq3);
 			err = sqlx_transaction_end(repctx, err);
 			if (!err)
-				meta2_backend_add_modified_container(m2b, sq3);
+				m2b_add_modified_container(m2b, sq3);
 		}
 		m2b_close(sq3);
-
-		namespace_info_free(args.nsinfo);
 	}
 
 	return err;
@@ -932,9 +937,7 @@ meta2_backend_copy_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 		memset(&args, 0, sizeof(args));
 		args.sq3 = sq3;
 		args.url = url;
-		args.max_versions = _maxvers(sq3, m2b);
-		args.nsinfo = meta2_backend_get_nsinfo(m2b);
-		args.lbpool = m2b->lb;
+		args.ns_max_versions = m2b_max_versions(m2b);
 
 		if (!(err = _transaction_begin(sq3, url, &repctx))) {
 			if (!(err = m2db_copy_alias(&args, src)))
@@ -942,8 +945,65 @@ meta2_backend_copy_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 			err = sqlx_transaction_end(repctx, err);
 		}
 		m2b_close(sq3);
+	}
 
-		namespace_info_free(args.nsinfo);
+	return err;
+}
+
+GError *
+meta2_backend_update_content(struct meta2_backend_s *m2b, struct oio_url_s *url,
+		GSList *in, GSList **out_deleted, GSList **out_added)
+{
+	GError *err = NULL;
+	struct sqlx_sqlite3_s *sq3 = NULL;
+	struct sqlx_repctx_s *repctx = NULL;
+
+	EXTRA_ASSERT(m2b != NULL);
+	EXTRA_ASSERT(url != NULL);
+	if (!in)
+		return NEWERROR(CODE_BAD_REQUEST, "No bean");
+
+	err = m2b_open(m2b, url, M2V2_OPEN_MASTERONLY|M2V2_OPEN_ENABLED, &sq3);
+	if (!err) {
+		if (!(err = _transaction_begin(sq3, url, &repctx))) {
+			if (!(err = m2db_update_content(sq3, url, in,
+						out_deleted, out_added)))
+				m2db_increment_version(sq3);
+			err = sqlx_transaction_end(repctx, err);
+			if (!err)
+				m2b_add_modified_container(m2b, sq3);
+		}
+		m2b_close(sq3);
+	}
+
+	return err;
+}
+
+GError *
+meta2_backend_truncate_content(struct meta2_backend_s *m2b,
+		struct oio_url_s *url, gint64 truncate_size,
+		GSList **out_deleted, GSList **out_added)
+{
+	GError *err = NULL;
+	struct sqlx_sqlite3_s *sq3 = NULL;
+	struct sqlx_repctx_s *repctx = NULL;
+
+	EXTRA_ASSERT(m2b != NULL);
+	EXTRA_ASSERT(url != NULL);
+	if (truncate_size < 0)
+		return NEWERROR(CODE_BAD_REQUEST, "Negative truncate size!");
+
+	err = m2b_open(m2b, url, M2V2_OPEN_MASTERONLY|M2V2_OPEN_ENABLED, &sq3);
+	if (!err) {
+		if (!(err = _transaction_begin(sq3, url, &repctx))) {
+			if (!(err = m2db_truncate_content(sq3, url, truncate_size,
+						out_deleted, out_added)))
+				m2db_increment_version(sq3);
+			err = sqlx_transaction_end(repctx, err);
+			if (!err)
+				m2b_add_modified_container(m2b, sq3);
+		}
+		m2b_close(sq3);
 	}
 
 	return err;
@@ -969,9 +1029,7 @@ meta2_backend_force_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 		memset(&args, 0, sizeof(args));
 		args.sq3 = sq3;
 		args.url = url;
-		args.max_versions = _maxvers(sq3, m2b);
-		args.nsinfo = meta2_backend_get_nsinfo(m2b);
-		args.lbpool = m2b->lb;
+		args.ns_max_versions = m2b_max_versions(m2b);
 
 		if (!(err = _transaction_begin(sq3,url, &repctx))) {
 			if (!(err = m2db_force_alias(&args, in, out_deleted, out_added)))
@@ -979,8 +1037,7 @@ meta2_backend_force_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 			err = sqlx_transaction_end(repctx, err);
 		}
 		if (!err)
-			meta2_backend_add_modified_container(m2b, sq3);
-		namespace_info_free(args.nsinfo);
+			m2b_add_modified_container(m2b, sq3);
 
 		m2b_close(sq3);
 	}
@@ -990,11 +1047,12 @@ meta2_backend_force_alias(struct meta2_backend_s *m2b, struct oio_url_s *url,
 
 GError*
 meta2_backend_insert_beans(struct meta2_backend_s *m2b,
-		struct oio_url_s *url, GSList *beans)
+		struct oio_url_s *url, GSList *beans, gboolean force)
 {
 	GError *err = NULL;
 	struct sqlx_sqlite3_s *sq3 = NULL;
 	struct sqlx_repctx_s *repctx = NULL;
+	int error_already = 0;
 
 	EXTRA_ASSERT(m2b != NULL);
 	EXTRA_ASSERT(url != NULL);
@@ -1002,14 +1060,29 @@ meta2_backend_insert_beans(struct meta2_backend_s *m2b,
 	err = m2b_open(m2b, url, M2V2_OPEN_MASTERONLY|M2V2_OPEN_ENABLED, &sq3);
 	if (!err) {
 		if (!(err = _transaction_begin(sq3, url, &repctx))) {
-			err = _db_save_beans_list (sq3->db, beans);
+			if (force)
+				err = _db_save_beans_list (sq3->db, beans);
+			else
+				err = _db_insert_beans_list (sq3->db, beans);
 			if (!err)
 				m2db_increment_version(sq3);
+			else {
+				/* A constraint error is usually raised by an actual constraint
+				 * violation in the DB (inside the transaction) or also by a
+				 * failure of the commit hook (on the final COMMIT). */
+				error_already |= (err->code == SQLITE_CONSTRAINT);
+			}
 			err = sqlx_transaction_end(repctx, err);
 		}
 		m2b_close(sq3);
 	}
 
+	if (error_already) {
+		EXTRA_ASSERT(err != NULL);
+		g_clear_error(&err);
+		err = NEWERROR(CODE_CONTENT_EXISTS, "Bean already present");
+	}
+
 	return err;
 }
 
@@ -1068,7 +1141,7 @@ meta2_backend_delete_beans(struct meta2_backend_s *m2b,
 }
 
 GError*
-meta2_backend_update_beans(struct meta2_backend_s *m2b, struct oio_url_s *url, 
+meta2_backend_update_beans(struct meta2_backend_s *m2b, struct oio_url_s *url,
 		GSList *new_chunks, GSList *old_chunks)
 {
 	GError *err = NULL;
@@ -1092,20 +1165,7 @@ meta2_backend_update_beans(struct meta2_backend_s *m2b, struct oio_url_s *url,
 		if (!(err = _transaction_begin(sq3, url, &repctx))) {
 			for (GSList *l0=old_chunks, *l1=new_chunks;
 					!err && l0 && l1 ; l0=l0->next,l1=l1->next)
-			{
-				err = _db_delete_bean (sq3->db, l0->data);
-				if (!err)
-					err = _db_save_bean (sq3->db, l1->data);
-				if (!err && DESCR(l0->data) == &descr_struct_CHUNKS) {
-					gchar *stmt = g_strdup_printf(
-							"UPDATE chunks SET id = '%s' WHERE id = '%s'",
-							CHUNKS_get_id(l1->data)->str, CHUNKS_get_id(l0->data)->str);
-					int rc = sqlx_exec(sq3->db, stmt);
-					g_free(stmt);
-					if (!sqlx_code_good(rc))
-						err = SQLITE_GERROR(sq3->db, rc);
-				}
-			}
+				err = _db_substitute_bean(sq3->db, l0->data, l1->data);
 			if (!err)
 				m2db_increment_version(sq3);
 			err = sqlx_transaction_end(repctx, err);
@@ -1124,8 +1184,7 @@ meta2_backend_get_alias_version(struct meta2_backend_s *m2b,
 	EXTRA_ASSERT(url != NULL);
 
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	GError *err = m2b_open(m2b, url, M2V2_OPEN_MASTERSLAVE
-			|M2V2_OPEN_ENABLED|M2V2_OPEN_FROZEN, &sq3);
+	GError *err = m2b_open(m2b, url, _mode_readonly(0), &sq3);
 	if (!err) {
 		err = m2db_get_alias_version(sq3, url, version);
 		m2b_close(sq3);
@@ -1142,7 +1201,6 @@ meta2_backend_append_to_alias(struct meta2_backend_s *m2b,
 	struct sqlx_sqlite3_s *sq3 = NULL;
 	struct sqlx_repctx_s *repctx = NULL;
 	struct namespace_info_s *nsinfo = NULL;
-	gint64 max_versions;
 
 	EXTRA_ASSERT(m2b != NULL);
 	EXTRA_ASSERT(url != NULL);
@@ -1153,14 +1211,13 @@ meta2_backend_append_to_alias(struct meta2_backend_s *m2b,
 
 	err = m2b_open(m2b, url, M2V2_OPEN_MASTERONLY|M2V2_OPEN_ENABLED, &sq3);
 	if (!err) {
-		max_versions = _maxvers(sq3, m2b);
 		if (!(err = _transaction_begin(sq3, url, &repctx))) {
-			if (!(err = m2db_append_to_alias(sq3, nsinfo, max_versions, url, beans, cb, u0)))
+			if (!(err = m2db_append_to_alias(sq3, url, beans, cb, u0)))
 				m2db_increment_version(sq3);
 			err = sqlx_transaction_end(repctx, err);
 		}
 		if (!err)
-			meta2_backend_add_modified_container(m2b, sq3);
+			m2b_add_modified_container(m2b, sq3);
 		m2b_close(sq3);
 	}
 
@@ -1170,14 +1227,14 @@ meta2_backend_append_to_alias(struct meta2_backend_s *m2b,
 
 GError*
 meta2_backend_get_properties(struct meta2_backend_s *m2b,
-		struct oio_url_s *url, m2_onbean_cb cb, gpointer u0)
+		struct oio_url_s *url, guint32 flags,
+		m2_onbean_cb cb, gpointer u0)
 {
 	EXTRA_ASSERT(m2b != NULL);
 	EXTRA_ASSERT(url != NULL);
 
 	struct sqlx_sqlite3_s *sq3 = NULL;
-	GError *err = m2b_open(m2b, url, M2V2_OPEN_MASTERSLAVE
-			|M2V2_OPEN_ENABLED|M2V2_OPEN_FROZEN, &sq3);
+	GError *err = m2b_open(m2b, url, _mode_readonly(flags), &sq3);
 	if (!err) {
 		err = m2db_get_properties(sq3, url, cb, u0);
 		m2b_close(sq3);
@@ -1314,6 +1371,10 @@ _meta2_backend_force_prepare_data_unlocked(struct meta2_backend_s *m2b,
 		memcpy(pdata_out, pdata, sizeof(struct m2_prepare_data));
 }
 
+/**
+ * Update the data structure allowing to answer PREPARE requests
+ * without taking the lock on the database file.
+ */
 static void
 _meta2_backend_force_prepare_data(struct meta2_backend_s *m2b,
 		const gchar *key, struct sqlx_sqlite3_s *sq3)
@@ -1329,15 +1390,20 @@ meta2_backend_change_callback(struct sqlx_sqlite3_s *sq3,
 {
 	gchar *account = sqlx_admin_get_str(sq3, SQLX_ADMIN_ACCOUNT);
 	gchar *user = sqlx_admin_get_str(sq3, SQLX_ADMIN_USERNAME);
-	struct oio_url_s *url = oio_url_empty();
-	oio_url_set(url, OIOURL_NS, m2b->ns_name);
-	oio_url_set(url, OIOURL_ACCOUNT, account);
-	oio_url_set(url, OIOURL_USER, user);
+	if (!account || !user) {
+		GRID_WARN("Missing "SQLX_ADMIN_ACCOUNT" or "SQLX_ADMIN_USERNAME
+				" in database %s", sq3->path);
+	} else {
+		struct oio_url_s *url = oio_url_empty();
+		oio_url_set(url, OIOURL_NS, m2b->ns_name);
+		oio_url_set(url, OIOURL_ACCOUNT, account);
+		oio_url_set(url, OIOURL_USER, user);
 
-	_meta2_backend_force_prepare_data(m2b,
-			oio_url_get(url, OIOURL_HEXID), sq3);
+		_meta2_backend_force_prepare_data(m2b,
+				oio_url_get(url, OIOURL_HEXID), sq3);
 
-	oio_url_clean(url);
+		oio_url_clean(url);
+	}
 	g_free(user);
 	g_free(account);
 }
@@ -1352,7 +1418,7 @@ meta2_backend_change_callback(struct sqlx_sqlite3_s *sq3,
  * @param sq3 output database pointer, in case we were forced to open it
  */
 static GError*
-_meta2_backend_get_prepare_data(struct meta2_backend_s *m2b,
+m2b_get_prepare_data(struct meta2_backend_s *m2b,
 		struct oio_url_s *url, struct m2_prepare_data *pdata_out,
 		struct sqlx_sqlite3_s **sq3)
 {
@@ -1370,8 +1436,7 @@ _meta2_backend_get_prepare_data(struct meta2_backend_s *m2b,
 		// Prepare data is not available. Open the base, take the writer lock
 		// and check again, in case another thread did the job while we were
 		// waiting for the base or the writer lock.
-		err = m2b_open(m2b, url,
-				M2V2_OPEN_MASTERSLAVE|M2V2_OPEN_ENABLED, sq3);
+		err = m2b_open(m2b, url, _mode_readonly(0), sq3);
 		if (!err) {
 			g_rw_lock_writer_lock(&(m2b->prepare_data_lock));
 			pdata = g_hash_table_lookup(m2b->prepare_data_cache, key);
@@ -1397,7 +1462,6 @@ meta2_backend_generate_beans(struct meta2_backend_s *m2b,
 	GError *err = NULL;
 	struct namespace_info_s *nsinfo;
 	struct storage_policy_s *policy = NULL;
-	struct grid_lb_iterator_s *iter = NULL;
 	struct m2_prepare_data pdata = {0};
 
 	GRID_TRACE("BEANS(%s,%"G_GINT64_FORMAT",%s)", oio_url_get(url, OIOURL_WHOLE),
@@ -1411,12 +1475,12 @@ meta2_backend_generate_beans(struct meta2_backend_s *m2b,
 
 	/* Get the data needed for the beans preparation.
 	 * This call may return an open database. */
-	_meta2_backend_get_prepare_data(m2b, url, &pdata, &sq3);
+	m2b_get_prepare_data(m2b, url, &pdata, &sq3);
 
 	if (m2b->flag_precheck_on_generate &&
 			VERSIONS_DISABLED(pdata.max_versions)) {
 		err = m2b_open_if_needed(m2b, url,
-				M2V2_OPEN_MASTERSLAVE|M2V2_OPEN_ENABLED, &sq3);
+				_mode_masterslave(0)|M2V2_OPEN_ENABLED, &sq3);
 		if (!err) {
 			/* If the versioning is not supported, we check the content
 			 * is not present */
@@ -1439,7 +1503,7 @@ meta2_backend_generate_beans(struct meta2_backend_s *m2b,
 			/* When appending, we must get the storage policy of
 			 * the existing content, thus we must open the base. */
 			err = m2b_open_if_needed(m2b, url,
-					M2V2_OPEN_MASTERSLAVE|M2V2_OPEN_ENABLED, &sq3);
+					_mode_masterslave(0)|M2V2_OPEN_ENABLED, &sq3);
 			if (!err)
 				err = m2db_get_storage_policy(sq3, url, nsinfo, append,
 						&policy);
@@ -1474,13 +1538,9 @@ meta2_backend_generate_beans(struct meta2_backend_s *m2b,
 
 	/* Let's continue to generate the beans, no need for an open container for the moment */
 	if (!err) {
-		iter = grid_lbpool_get_iterator(m2b->lb, "rawx");
-		if (!iter)
-			err = NEWERROR(CODE_POLICY_NOT_SATISFIABLE, "No RAWX available");
-		else
-			err = m2_generate_beans(url, size,
-					namespace_chunk_size(nsinfo, oio_url_get(url, OIOURL_NS)),
-					policy, iter, cb, cb_data);
+		err = m2_generate_beans(url, size,
+				namespace_chunk_size(nsinfo, oio_url_get(url, OIOURL_NS)),
+				policy, m2b->lb, cb, cb_data);
 	}
 
 	namespace_info_free(nsinfo);
@@ -1488,69 +1548,6 @@ meta2_backend_generate_beans(struct meta2_backend_s *m2b,
 	return err;
 }
 
-// TODO FIXME too many arguments
-// TODO 'url' seems only useful for logging purposes
-GError*
-meta2_backend_get_conditionned_spare_chunks(struct meta2_backend_s *m2b,
-		struct oio_url_s *url, gint64 count, gint64 dist, const char *notin,
-		const char *broken, GSList **result)
-{
-	GError *err = NULL;
-	GSList *notin2 = NULL;
-	GSList *broken2 = NULL;
-	struct storage_policy_s *stgpol = NULL;
-
-	GSList * srvinfo_from_piped_chunkid(const char *str)
-	{
-		GError *err2 = NULL;
-		GSList *sil = NULL;
-
-		if(!str || strlen(str) <= 0)
-			return NULL;
-
-		char **urls = g_strsplit(str, "|", 0);
-		for (uint i = 0; i < g_strv_length(urls); i++) {
-			if (strlen(urls[i]) <= 0)
-				continue;
-			struct service_info_s *si = NULL;
-			err2 = service_info_from_chunk_id(m2b->lb, urls[i], &si);
-			if (NULL != si)
-				sil = g_slist_prepend(sil, si);
-			if (err2 != NULL) {
-				GRID_WARN("Failed getting service info from '%s': %s",
-						urls[i], err2->message);
-				g_clear_error(&err2);
-			}
-		}
-
-		g_strfreev(urls);
-		return sil;
-	}
-
-	(void) url;
-	GRID_TRACE("CONDITIONNED SPARE(%s, %"G_GINT64_FORMAT", %"G_GINT64_FORMAT", %s, %s)",
-			oio_url_get(url, OIOURL_WHOLE),
-			count,
-			dist,
-			notin,
-			broken);
-
-	notin2 = srvinfo_from_piped_chunkid(notin);
-	broken2 = srvinfo_from_piped_chunkid(broken);
-
-	// FIXME: storage class should come as parameter
-	stgpol = storage_policy_init(m2b->nsinfo, NULL);
-
-	err = get_conditioned_spare_chunks(m2b->lb, count, dist,
-			storage_policy_get_storage_class(stgpol), notin2, broken2, result);
-
-	g_slist_free_full(notin2, (GDestroyNotify) service_info_gclean);
-	g_slist_free_full(broken2, (GDestroyNotify) service_info_gclean);
-	storage_policy_clean(stgpol);
-
-	return err;
-}
-
 static GError*
 _load_storage_policy(struct meta2_backend_s *m2b, struct oio_url_s *url,
 		const gchar *polname, struct storage_policy_s **pol)
@@ -1597,7 +1594,9 @@ meta2_backend_get_conditionned_spare_chunks_v2(struct meta2_backend_s *m2b,
 	struct storage_policy_s *pol = NULL;
 	GError *err = _load_storage_policy(m2b, url, polname, &pol);
 	if (!err)
-		err = get_conditioned_spare_chunks2(m2b->lb, pol, notin, broken, result);
+		err = get_conditioned_spare_chunks(m2b->lb,
+				storage_policy_get_service_pool(pol),
+				m2b->ns_name, notin, broken, result);
 	if (pol)
 		storage_policy_clean(pol);
 	return err;
@@ -1610,10 +1609,13 @@ meta2_backend_get_spare_chunks(struct meta2_backend_s *m2b, struct oio_url_s *ur
 	GRID_TRACE("SPARE(%s,%s)", oio_url_get(url, OIOURL_WHOLE), polname);
 	EXTRA_ASSERT(m2b != NULL);
 
+	// TODO: we can avoid instantiating storage policy
+	// but we need to review several function calls
 	struct storage_policy_s *pol = NULL;
 	GError *err = _load_storage_policy(m2b, url, polname, &pol);
 	if (!err)
-		err = get_spare_chunks(m2b->lb, pol, result);
+		err = get_spare_chunks(m2b->lb, storage_policy_get_service_pool(pol),
+				result);
 	if (pol)
 		storage_policy_clean(pol);
 	return err;
@@ -1632,8 +1634,7 @@ meta2_backend_content_from_chunkid(struct meta2_backend_s *m2b,
 	EXTRA_ASSERT(m2b != NULL);
 	EXTRA_ASSERT(url != NULL);
 
-	err = m2b_open(m2b, url, M2V2_OPEN_MASTERSLAVE|
-			M2V2_OPEN_ENABLED|M2V2_OPEN_FROZEN, &sq3);
+	err = m2b_open(m2b, url, _mode_readonly(0), &sq3);
 	if (!err) {
 		GVariant *params[2] = {NULL, NULL};
 		params[0] = g_variant_new_string(chunk_id);
@@ -1661,8 +1662,7 @@ meta2_backend_content_from_contenthash (struct meta2_backend_s *m2b,
 	EXTRA_ASSERT(m2b != NULL);
 	EXTRA_ASSERT(url != NULL);
 
-	err = m2b_open(m2b, url, M2V2_OPEN_MASTERSLAVE|
-			M2V2_OPEN_ENABLED|M2V2_OPEN_FROZEN, &sq3);
+	err = m2b_open(m2b, url, _mode_readonly(0), &sq3);
 	if (!err) {
 		GVariant *params[2] = {NULL, NULL};
 		params[0] = _gb_to_gvariant(h);
@@ -1688,8 +1688,7 @@ meta2_backend_content_from_contentid (struct meta2_backend_s *m2b,
 	EXTRA_ASSERT(m2b != NULL);
 	EXTRA_ASSERT(url != NULL);
 
-	err = m2b_open(m2b, url, M2V2_OPEN_MASTERSLAVE|
-			M2V2_OPEN_ENABLED|M2V2_OPEN_FROZEN, &sq3);
+	err = m2b_open(m2b, url, _mode_readonly(0), &sq3);
 	if (!err) {
 		GVariant *params[2] = {NULL, NULL};
 		params[0] = _gb_to_gvariant(h);
