@@ -158,7 +158,8 @@ dav_rawx_get_resource(request_rec *r, const char *root_dir, const char *label,
 		if (!authorized_personal_only(r->connection->remote_ip, conf->rawx_conf->acl))
 #endif
 		{
-			return server_create_and_stat_error(conf, r->pool, HTTP_UNAUTHORIZED, 0, "Permission Denied (APO)");
+			return server_create_and_stat_error(conf, r->pool,
+					HTTP_UNAUTHORIZED, 0, "Permission Denied (APO)");
 		}
 	}
 
@@ -167,9 +168,11 @@ dav_rawx_get_resource(request_rec *r, const char *root_dir, const char *label,
 	ctx.pool = r->pool;
 	ctx.request = r;
 
-	dav_error *e = rawx_repo_check_request(r, root_dir, label, use_checked_in, &ctx, result_resource);
-	/* Return in case we have an error or if result_resource != null because it was an info request */
-	if (NULL != e || NULL != *result_resource) {
+	dav_error *e = rawx_repo_check_request(r, root_dir, label, use_checked_in,
+			&ctx, result_resource);
+	/* Return in case we have an error or
+	 * if result_resource != null because it was an info request */
+	if (e || *result_resource) {
 		return e;
 	}
 
@@ -183,10 +186,9 @@ dav_rawx_get_resource(request_rec *r, const char *root_dir, const char *label,
 			return e;
 		}
 	}
-	DAV_DEBUG_REQ(r, 0, "Hashed directory : %.*s", (int)sizeof(ctx.dirname), ctx.dirname);
-
-	/* All the checks on the URL have been passed, now build a resource */
+	DAV_DEBUG_REQ(r, 0, "Hashed directory: %.*s", (int)sizeof(ctx.dirname), ctx.dirname);
 
+	/* All the checks on the URL have passed, now build a resource */
 	dav_resource *resource = apr_pcalloc(r->pool, sizeof(*resource));
 	resource->type = DAV_RESOURCE_TYPE_REGULAR;
 	resource->info = apr_pcalloc(r->pool, sizeof(ctx));;
@@ -200,8 +202,7 @@ dav_rawx_get_resource(request_rec *r, const char *root_dir, const char *label,
 		resource->info->file_extension,
 		NULL);
 
-	/* init compression context structure if we are in get method (for decompression) */
-
+	/* init compression context structure if we are in get method */
 	if (r->method_number == M_GET && !ctx.update_only) {
 		resource_init_decompression(resource, conf);
 	}
@@ -219,16 +220,17 @@ dav_rawx_get_resource(request_rec *r, const char *root_dir, const char *label,
 	if (r->method_number == M_PUT || r->method_number == M_POST ||
 			r->method_number == M_MOVE ||
 			(r->method_number == M_GET && ctx.update_only)) {
-		const char *missing = request_load_chunk_info(r, resource);
+		request_load_chunk_info_from_headers(r, &(resource->info->chunk));
+		const char *missing = check_chunk_info(&resource->info->chunk);
 		if (missing != NULL) {
 			return server_create_and_stat_error(request_get_server_config(r), r->pool,
-				HTTP_BAD_REQUEST, 0, apr_pstrcat(r->pool, "Missing header ", missing, NULL));
+				HTTP_BAD_REQUEST, 0, apr_pstrcat(r->pool, "missing or invalid header ", missing, NULL));
 		}
 	}
 
 	if (r->method_number == M_POST || r->method_number == M_PUT) {
-		if (resource->info->chunk.id) {
-			if (0 != apr_strnatcasecmp(resource->info->chunk.id, resource->info->hex_chunkid))
+		if (resource->info->chunk.chunk_id) {
+			if (0 != apr_strnatcasecmp(resource->info->chunk.chunk_id, resource->info->hex_chunkid))
 				return server_create_and_stat_error(request_get_server_config(r), r->pool,
 						HTTP_BAD_REQUEST, 0, "chunk-id mismatch");
 		}
@@ -316,6 +318,16 @@ dav_rawx_open_stream(const dav_resource *resource, dav_stream_mode mode, dav_str
 	return NULL;
 }
 
+static void
+_rollback_and_log(dav_stream *stream)
+{
+	dav_error *e = rawx_repo_rollback_upload(stream);
+	if (!e)
+		return;
+	DAV_ERROR_REQ(stream->r->info->request, 0,
+			"Error while rolling back upload: %s", e->desc);
+}
+
 static dav_error *
 dav_rawx_close_stream(dav_stream *stream, int commit)
 {
@@ -323,23 +335,22 @@ dav_rawx_close_stream(dav_stream *stream, int commit)
 
 	dav_error *e = NULL;
 
-	DAV_DEBUG_REQ(stream->r->info->request, 0, "Closing (%s) the stream to [%s]",
-		(commit ? "commit" : "rollback"), stream->pathname);
+	DAV_DEBUG_REQ(stream->r->info->request, 0,
+			"Closing (%s) the stream to [%s]",
+			(commit ? "commit" : "rollback"), stream->pathname);
 
 	if (!commit) {
 		e = rawx_repo_rollback_upload(stream);
 	} else {
 		e = rawx_repo_write_last_data_crumble(stream);
-		if ( NULL != e ) {
-			DAV_DEBUG_REQ(stream->r->info->request, 0, "Cannot commit, an error occured while writing end of data");
-			/* Must we did it ? */
-			dav_error *e_tmp = NULL;
-			e_tmp = rawx_repo_rollback_upload(stream);
-			if ( NULL != e_tmp) {
-				DAV_ERROR_REQ(stream->r->info->request, 0, "Error while rolling back upload : %s", e_tmp->desc);
-			}
+		if (e) {
+			DAV_DEBUG_REQ(stream->r->info->request, 0,
+					"Cannot commit, an error occured while writing end of data");
+			_rollback_and_log(stream);
 		} else {
 			e = rawx_repo_commit_upload(stream);
+			if (e)
+				_rollback_and_log(stream);
 		}
 	}
 
@@ -354,7 +365,7 @@ dav_rawx_close_stream(dav_stream *stream, int commit)
 			request_get_duration(stream->r->info->request));
 
 	if (stream->md5) {
-		g_checksum_free (stream->md5);
+		g_checksum_free(stream->md5);
 		stream->md5 = NULL;
 	}
 	return e;
@@ -365,21 +376,23 @@ dav_rawx_write_stream(dav_stream *stream, const void *buf, apr_size_t bufsize)
 {
 	DAV_XDEBUG_POOL(stream->p, 0, "%s(%s)", __FUNCTION__, stream->pathname);
 
-	guint written = 0;
+	apr_size_t copied = 0;
 	gulong checksum = stream->compress_checksum;
 
-	while (written < bufsize) {
-		memcpy(stream->buffer + stream->bufsize, buf + written, MIN(bufsize - written, stream->blocksize - stream->bufsize));
-		guint tmp = MIN(bufsize - written, stream->blocksize - stream->bufsize);
-		written += tmp;
-		stream->bufsize += tmp;
+	while (copied < bufsize) {
+		apr_size_t to_copy = MIN(bufsize - copied,
+				stream->buffer_size - stream->buffer_offset);
+		memcpy(stream->buffer + stream->buffer_offset, buf + copied, to_copy);
+		copied += to_copy;
+		stream->buffer_offset += to_copy;
 
 		/* If buffer full, compress if needed and write to distant file */
-		if (stream->blocksize - stream->bufsize <=0){
-			gsize nb_write = 0;
+		if (stream->buffer_size - stream->buffer_offset <= 0){
+			size_t written = 0;
 			if (!stream->compression) {
-				nb_write = fwrite(stream->buffer, stream->bufsize, 1, stream->f);
-				if (nb_write != 1) {
+				written = fwrite(stream->buffer, stream->buffer_offset, 1,
+						stream->f);
+				if (written != 1) {
 					/* ### use something besides 500? */
 					return server_create_and_stat_error(resource_get_server_config(stream->r), stream->p,
 							HTTP_INTERNAL_SERVER_ERROR, 0,
@@ -388,7 +401,7 @@ dav_rawx_write_stream(dav_stream *stream, const void *buf, apr_size_t bufsize)
 				}
 			} else {
 				GByteArray *gba = g_byte_array_new();
-				if (stream->comp_ctx.data_compressor(stream->buffer, stream->bufsize, gba,
+				if (stream->comp_ctx.data_compressor(stream->buffer, stream->buffer_offset, gba,
 							&checksum)!=0) {
 					if (gba)
 						g_byte_array_free(gba, TRUE);
@@ -397,8 +410,8 @@ dav_rawx_write_stream(dav_stream *stream, const void *buf, apr_size_t bufsize)
 							HTTP_INTERNAL_SERVER_ERROR, 0,
 							"An error occurred while compressing data.");
 				}
-				nb_write = fwrite(gba->data, gba->len, 1, stream->f);
-				if (nb_write != 1) {
+				written = fwrite(gba->data, gba->len, 1, stream->f);
+				if (written != 1) {
 					if (gba)
 						g_byte_array_free(gba, TRUE);
 					/* ### use something besides 500? */
@@ -412,15 +425,17 @@ dav_rawx_write_stream(dav_stream *stream, const void *buf, apr_size_t bufsize)
 					g_byte_array_free(gba, TRUE);
 			}
 
-			stream->buffer = apr_pcalloc(stream->p, stream->blocksize);
-			stream->bufsize = 0;
+			memset(stream->buffer, 0, stream->buffer_size);
+			stream->buffer_offset = 0;
 		}
 	}
 
 	stream->compress_checksum = checksum;
 
 	/* update the hash and the stats */
-	g_checksum_update(stream->md5, buf, bufsize);
+	if (stream->md5)
+		g_checksum_update(stream->md5, buf, bufsize);
+
 	/* update total_size */
 	stream->total_size += bufsize;
 	return NULL;
@@ -464,7 +479,7 @@ dav_rawx_set_headers(request_rec *r, const dav_resource *resource)
 	/* set up the Content-Length header */
 	ap_set_content_length(r, resource->info->finfo.size);
 
-	request_fill_headers(r, &(resource->info->content), &(resource->info->chunk));
+	request_fill_headers(r, &(resource->info->chunk));
 
 	/* compute metadata_compress if compressed content */
 	if (resource->info->compression) {
@@ -483,7 +498,6 @@ dav_rawx_deliver(const dav_resource *resource, ap_filter_t *output)
 	apr_pool_t *pool;
 	apr_bucket_brigade *bb = NULL;
 	apr_status_t status;
-	apr_bucket *bkt = NULL;
 	dav_resource_private *ctx;
 	dav_error *e = NULL;
 
@@ -525,8 +539,7 @@ dav_rawx_deliver(const dav_resource *resource, ap_filter_t *output)
 		FILE *f = NULL;
 		f = fopen(path, "r");
 		/* Try to open the file but forbids a creation */
-		if (!set_rawx_full_info_in_attr(path, fileno(f), &error_local,&(ctx->content),
-					&(ctx->chunk), NULL, NULL)) {
+		if (!set_rawx_info_to_fd(fileno(f), &error_local, &(ctx->chunk))) {
 			fclose(f);
 			e = server_create_and_stat_error(conf, pool,
 					HTTP_FORBIDDEN, 0, apr_pstrdup(pool, gerror_get_message(error_local)));
@@ -542,17 +555,19 @@ dav_rawx_deliver(const dav_resource *resource, ap_filter_t *output)
 
 			/* Try to open the file but forbids a creation */
 			status = apr_file_open(&fd, resource_get_pathname(resource),
-					APR_READ|APR_BINARY|APR_BUFFERED, 0, pool);
+					APR_FOPEN_READ|
+					APR_FOPEN_BINARY|
+					APR_FOPEN_BUFFERED|
+					APR_FOPEN_SENDFILE_ENABLED,
+					0, pool);
 			if (APR_SUCCESS != status) {
 				e = server_create_and_stat_error(conf, pool, HTTP_FORBIDDEN,
 						0, "File permissions deny server access.");
 				goto end_deliver;
 			}
 
-			/* FIXME this does not handle large files. but this is test code anyway */
-			bkt = apr_bucket_file_create(fd, 0,
-					(apr_size_t)resource->info->finfo.size,
-					pool, output->c->bucket_alloc);
+			apr_brigade_insert_file(bb, fd, 0,
+					(apr_size_t)resource->info->finfo.size, pool);
 		}
 		else {
 			DAV_DEBUG_RES(resource, 0, "Building a compressed resource bucket");
@@ -561,23 +576,24 @@ dav_rawx_deliver(const dav_resource *resource, ap_filter_t *output)
 			i64 = g_ascii_strtoll(ctx->cp_chunk.uncompressed_size, NULL, 10);
 
 			/* creation of compression specific bucket */
-			bkt = apr_pcalloc(pool, sizeof(struct apr_bucket));
+			apr_bucket *bkt = apr_pcalloc(pool, sizeof(struct apr_bucket));
 			bkt->type = &chunk_bucket_type;
 			bkt->length = i64;
 			bkt->start = 0;
 			bkt->data = ctx;
 			bkt->free = chunk_bucket_free_noop;
 			bkt->list = output->c->bucket_alloc;
+
+			APR_BRIGADE_INSERT_TAIL(bb, bkt);
 		}
 
-		APR_BRIGADE_INSERT_TAIL(bb, bkt);
-
 		/* as soon as the chunk has been sent, end of stream!*/
-		bkt = apr_bucket_eos_create(output->c->bucket_alloc);
-		APR_BRIGADE_INSERT_TAIL(bb, bkt);
+		APR_BRIGADE_INSERT_TAIL(bb,
+				apr_bucket_eos_create(output->c->bucket_alloc));
 
 		if ((status = ap_pass_brigade(output, bb)) != APR_SUCCESS){
-			e = server_create_and_stat_error(conf, pool, HTTP_FORBIDDEN, 0, "Could not write contents to filter.");
+			e = server_create_and_stat_error(conf, pool, HTTP_FORBIDDEN, 0,
+					"Failed to send data to the client (timed out?)");
 			/* close file */
 			if (ctx->cp_chunk.fd) {
 				fclose(ctx->cp_chunk.fd);
@@ -997,8 +1013,8 @@ dav_rawx_patch_exec(const dav_resource *resource, const apr_xml_elem *elem, int
 	(void) operation;
 	(void) context;
 	(void) rollback_ctx;
-	/* XXX JFS : TODO dump the xattr handle in the file */
-	return __dav_new_error(resource->info->pool, HTTP_INTERNAL_SERVER_ERROR, 0, "PROPPATCH not yet implemented");
+	return __dav_new_error(resource->info->pool, HTTP_NOT_IMPLEMENTED,
+			0, "PROPPATCH not yet implemented");
 }
 
 static void
