@@ -273,12 +273,16 @@ sqlx_repository_init(const gchar *vol, const struct sqlx_repo_config_s *cfg,
 	struct stat s;
 	sqlx_repository_t *repo;
 
+	g_assert_nonnull(vol);
+	g_assert_nonnull(cfg);
+	g_assert_nonnull(result);
+
 	(void) sqlite3_initialize();
 
 	if (!sqlite3_threadsafe())
 		return NEWERROR(0, "SQLite not in safe mode");
 
-	if (cfg && cfg->flags & SQLX_REPO_NOCACHE) {
+	if (cfg->flags & SQLX_REPO_NOCACHE) {
 		/* if there are several connections on the same base, we will use a
 		   shared cache that wil prevent us of too many I/O operations. */
 		if (SQLITE_OK != sqlite3_enable_shared_cache(1))
@@ -318,25 +322,19 @@ sqlx_repository_init(const gchar *vol, const struct sqlx_repo_config_s *cfg,
 
 	repo->schemas = g_tree_new_full(metautils_strcmp3, NULL, g_free, g_free);
 
-	repo->flag_autocreate = !cfg ? TRUE : BOOL(cfg->flags & SQLX_REPO_AUTOCREATE);
-	repo->flag_autovacuum = !cfg ? FALSE : BOOL(cfg->flags & SQLX_REPO_VACUUM);
-	repo->flag_delete_on = !cfg ? FALSE : BOOL(cfg->flags & SQLX_REPO_DELETEON);
+	repo->flag_autocreate = BOOL(cfg->flags & SQLX_REPO_AUTOCREATE);
+	repo->flag_autovacuum = BOOL(cfg->flags & SQLX_REPO_VACUUM);
+	repo->flag_delete_on = BOOL(cfg->flags & SQLX_REPO_DELETEON);
 
-	if (!cfg || !(cfg->flags & SQLX_REPO_NOCACHE)) {
-		repo->cache = sqlx_cache_init();
+	if (!(cfg->flags & SQLX_REPO_NOCACHE)) {
+		repo->cache = sqlx_cache_init(cfg->max_bases ? cfg->max_bases : SQLX_MAX_BASES);
 		sqlx_cache_set_close_hook(repo->cache,
 				(sqlx_cache_close_hook)__close_base);
 	}
 
-	if (cfg) {
-		repo->sync_mode_solo = cfg->sync_solo;
-		repo->sync_mode_repli = cfg->sync_repli;
-		repo->page_size = cfg->page_size;
-	} else {
-		repo->sync_mode_solo = SQLX_SYNC_NORMAL;
-		repo->sync_mode_repli = SQLX_SYNC_OFF;
-		repo->page_size = SQLX_DEFAULT_PAGE_SIZE;
-	}
+	repo->sync_mode_solo = cfg->sync_solo;
+	repo->sync_mode_repli = cfg->sync_repli;
+	repo->page_size = cfg->page_size;
 
 	repo->locator = _default_locator;
 	repo->locator_data = NULL;
@@ -550,8 +548,8 @@ sqlx_repository_get_cache(struct sqlx_repository_s *r)
 const gchar*
 sqlx_repository_get_local_addr(struct sqlx_repository_s *repo)
 {
-    struct election_manager_s* em = sqlx_repository_get_elections_manager(repo);
-    if (em)
+	struct election_manager_s* em = sqlx_repository_get_elections_manager(repo);
+	if (em)
 		return election_manager_get_local (em);
 	return NULL;
 }
@@ -568,6 +566,7 @@ struct open_args_s
 
 	gboolean create : 1;
 	gboolean no_refcheck : 1;
+	gboolean urgent : 1;
 	gboolean is_replicated : 1;
 };
 
@@ -649,12 +648,11 @@ sqlx_admin_reload(struct sqlx_sqlite3_s *sq3)
 {
 	if (sq3->admin)
 		g_tree_destroy(sq3->admin);
-	sq3->admin = g_tree_new_full(metautils_strcmp3, NULL,
-			g_free, metautils_gba_unref);
+	sq3->admin = g_tree_new_full(metautils_strcmp3, NULL, g_free, g_free);
 
 	sqlx_admin_load (sq3);
 	sqlx_admin_ensure_versions (sq3);
-    sqlx_admin_save_lazy_tnx (sq3);
+	sqlx_admin_save_lazy_tnx (sq3);
 	sq3->admin_dirty = 0;
 	GRID_TRACE("Loaded %u ADMIN from [%s.%s]", g_tree_nnodes(sq3->admin),
 			sq3->name.base, sq3->name.type);
@@ -695,6 +693,7 @@ retry:
 				GRID_DEBUG("DB creation error on [%s] : (%d) %s",
 						args->realpath, error->code, error->message);
 			} else {
+			// FALLTHROUGH
 		default:
 				_close_handle(&handle);
 				GRID_DEBUG("Open strong error [%s] : (%d) %s",
@@ -722,8 +721,7 @@ retry:
 	sq3->name.ns = g_strdup(args->name.ns);
 	sq3->path = g_strdup(args->realpath);
 	sq3->admin_dirty = 0;
-	sq3->admin = g_tree_new_full(metautils_strcmp3, NULL,
-			g_free, metautils_gba_unref);
+	sq3->admin = g_tree_new_full(metautils_strcmp3, NULL, g_free, g_free);
 
 	sqlx_exec(handle, "PRAGMA foreign_keys = OFF");
 	sqlx_exec(handle, "PRAGMA journal_mode = MEMORY");
@@ -765,7 +763,8 @@ __open_maybe_cached(struct open_args_s *args, struct sqlx_sqlite3_s **result)
 	GError *e0;
 	gint bd = -1;
 
-	e0 = sqlx_cache_open_and_lock_base(args->repo->cache, args->realname, &bd);
+	e0 = sqlx_cache_open_and_lock_base(args->repo->cache, args->realname,
+		   args->urgent, &bd);
 	if (e0 != NULL) {
 		g_prefix_error(&e0, "cache error: ");
 		return e0;
@@ -977,6 +976,7 @@ sqlx_repository_open_and_lock(sqlx_repository_t *repo,
 		return err;
 	args.no_refcheck = BOOL(how & SQLX_OPEN_NOREFCHECK);
 	args.create = BOOL(how & SQLX_OPEN_CREATE);
+	args.urgent = BOOL(how & SQLX_OPEN_URGENT);
 
 	switch (how & SQLX_OPEN_REPLIMODE) {
 		case SQLX_OPEN_LOCAL:
@@ -1110,7 +1110,7 @@ sqlx_repository_status_base(sqlx_repository_t *repo, const struct sqlx_name_s *n
 				gboolean master_in_peers = FALSE;
 				GError *err2 = election_get_peers(repo->election_manager, n, FALSE, &my_peers);
 				for (gchar **cursor = my_peers;
-						cursor && *cursor && !master_in_peers;
+						my_peers && *cursor && !master_in_peers;
 						cursor++) {
 					master_in_peers |= (0 == g_strcmp0(url, *cursor));
 				}
